@@ -158,27 +158,31 @@ def date_to_string(d):
     return d.strftime("%Y-%m-%d%z")
 
 
-class InvoiceError(Exception):
+class PyXMLiError(Exception):
     pass
 
 
-class GroupError(Exception):
+class InvoiceError(PyXMLiError):
     pass
 
 
-class DeliveryMethodException(Exception):
+class GroupError(PyXMLiError):
     pass
 
 
-class PaymentError(Exception):
+class DeliveryMethodException(PyXMLiError):
     pass
 
 
-class LineError(Exception):
+class PaymentError(PyXMLiError):
     pass
 
 
-class TreatmentException(Exception):
+class LineError(PyXMLiError):
+    pass
+
+
+class TreatmentException(PyXMLiError):
     pass
 
 
@@ -546,12 +550,19 @@ class Invoice(ExtensibleXMLiElement):
         self.seller = seller
         self.buyer = buyer
         self.__shipping = shipping
-        self.name = name
-        self.description = description
-        self.currency = currency
+        self.__name = None
+        if name:
+            self.name = name
+        self.__description = None
+        if description:
+            self.description = description
+        self.__currency = None
+        if currency:
+            self.currency = currency
         self.status = status
         self.date = date
-        self.due_date = due_date or self.date
+        self.due_date = due_date or (self.date if type(self.date) == date else
+                                     date.date())
         self.terms = terms
         self.__deliveries = []
         self.__groups = []
@@ -979,8 +990,8 @@ class Payment(ExtensibleXMLiElement):
         root = doc.createElement("payment")
         super(Payment, self).to_xml(root)
         self._create_text_node(root, "amount", self.amount)
-        self._create_text_node(root, "reference", self.ref, True)
         self._create_text_node(root, "method", self.method)
+        self._create_text_node(root, "reference", self.ref, True)
         self._create_text_node(root, "date", self.date)
         return root
 
